# 1 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!--------------------------------------------------------------------------------------------------!
!   CP2K: A general program to perform molecular dynamics simulations                              !
!   Copyright 2000-2021 CP2K developers group <https://cp2k.org>                                   !
!                                                                                                  !
!   SPDX-License-Identifier: GPL-2.0-or-later                                                      !
!--------------------------------------------------------------------------------------------------!

# 1 "/builddir/build/BUILD/cp2k-9.1/src/fm/../common/cp_linked_list.fypp" 1
# 7 "/builddir/build/BUILD/cp2k-9.1/src/fm/../common/cp_linked_list.fypp"
# 610 "/builddir/build/BUILD/cp2k-9.1/src/fm/../common/cp_linked_list.fypp"
# 9 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F" 2

MODULE cp_linked_list_fm
   USE cp_fm_types, ONLY: cp_fm_type, cp_fm_p_type
#include "../base/base_uses.f90"

# 26 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \brief describes a generic linked list template.
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      Linked list are supposed to always use pointers to the nodes for
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      basically everything: a pointer to the node is a list, an element of
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      the list, an iterator between the elment of the list.
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      An empty list is represented by an unassociated pointer.
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \par History
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      1.2002 created
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      4.2002 changed most of the methods, by making access to the list
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>             always through pointers (identifying pointer, list and iterators)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      6.2004 removed %initialized from list elements
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \author Fawzi Mohamed
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      IMPLICIT NONE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      PRIVATE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      LOGICAL, PRIVATE, PARAMETER :: debug_this_module = .TRUE.
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!API type
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         PUBLIC :: cp_sll_fm_type, cp_sll_fm_p_type
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!API common methods
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      PUBLIC :: cp_create, cp_dealloc, cp_next
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!API special get
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      PUBLIC :: cp_get_first_el, cp_get_rest, cp_get_length, cp_get_element_at, cp_to_array
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!API special set
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      PUBLIC :: cp_set_element_at
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!API structure manipulation
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      PUBLIC :: cp_insert, cp_remove_first_el, cp_remove_el, cp_remove_all
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      !API underlying routines
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         PUBLIC :: cp_sll_fm_create, cp_sll_fm_dealloc, &
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                   cp_sll_fm_next, &
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                   cp_sll_fm_get_length, &
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                   cp_sll_fm_get_el_at, cp_sll_fm_set_el_at, &
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                   cp_sll_fm_get_first_el, cp_sll_fm_get_rest, &
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                   cp_sll_fm_insert_el, cp_sll_fm_insert_el_at, &
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                   cp_sll_fm_rm_first_el, cp_sll_fm_rm_el_at, &
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                   cp_sll_fm_rm_all_el, &
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                   cp_sll_fm_to_array
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! creation of an object (from a pointer)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      INTERFACE cp_create
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            MODULE PROCEDURE cp_sll_fm_create
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      END INTERFACE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! destruction of an object (from a pointer)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      INTERFACE cp_dealloc
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            MODULE PROCEDURE cp_sll_fm_dealloc
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      END INTERFACE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! iterates to the next element
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      INTERFACE cp_next
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            MODULE PROCEDURE cp_sll_fm_next
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      END INTERFACE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! returns the first element
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      INTERFACE cp_get_first_el
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            MODULE PROCEDURE cp_sll_fm_get_first_el
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      END INTERFACE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! returns the rest of the list
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      INTERFACE cp_get_rest
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            MODULE PROCEDURE cp_sll_fm_get_rest
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      END INTERFACE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! returns the length of the list
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      INTERFACE cp_get_length
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            MODULE PROCEDURE cp_sll_fm_get_length
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      END INTERFACE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! returns the element at the given position
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      INTERFACE cp_get_element_at
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            MODULE PROCEDURE cp_sll_fm_get_el_at
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      END INTERFACE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! sets the element at the given position
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      INTERFACE cp_set_element_at
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            MODULE PROCEDURE cp_sll_fm_set_el_at
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      END INTERFACE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! inserts one element call cp_insert(list,element,...)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      INTERFACE cp_insert
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            MODULE PROCEDURE cp_sll_fm_insert_el
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      END INTERFACE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      INTERFACE cp_insert_at
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            MODULE PROCEDURE cp_sll_fm_insert_el_at
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      END INTERFACE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! removes an element
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      INTERFACE cp_remove_el
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            MODULE PROCEDURE cp_sll_fm_rm_first_el, &
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               cp_sll_fm_rm_el_at
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      END INTERFACE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! removes the first el
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      INTERFACE cp_remove_first_el
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            MODULE PROCEDURE cp_sll_fm_rm_first_el
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      END INTERFACE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! remove all the elements
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      INTERFACE cp_remove_all
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            MODULE PROCEDURE cp_sll_fm_rm_all_el
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      END INTERFACE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! transorms the list in array
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      INTERFACE cp_to_array
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            module procedure cp_sll_fm_to_array
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      END INTERFACE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \brief represent a single linked list that stores pointers to the elements
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param first_el the element that is stored in this node.
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param rest the rest of the list
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param empty true if the list pointer is not associated, if it points to
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>             to a not it is always false (as there is at least the
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>             first_el in the list)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param length the number of elements in the list
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \note
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      List are alway accessed through pointers, so every node of the
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      linked list can be seen as a list, its first element
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      a pointer to the position before itself, in a very natural way:
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      all the insertions take place before the actual element, and
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      you still can insert an element at the end.
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      This way I could identify nodes, lists and pointers between the
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      elements of the list.
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      Indexing is 1 based.
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \par History
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      none
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \author Fawzi Mohamed
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         TYPE cp_sll_fm_type
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            type(cp_fm_type),pointer :: first_el
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            TYPE(cp_sll_fm_type), POINTER :: rest
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         END TYPE cp_sll_fm_type
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \brief pointer to a linked list (to make arrays of pointers)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param list the pointer to the list
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \par History
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      none
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \author Fawzi Mohamed
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         TYPE cp_sll_fm_p_type
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            TYPE(cp_sll_fm_type), POINTER :: list
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         END TYPE cp_sll_fm_p_type
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      CONTAINS
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
      
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! =========== creation / destruction ========
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \brief allocates and initializes a single linked list
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param sll the single linked list to initialize
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param first_el the first element of this list
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param rest the following elements (if not given: empty)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \par History
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      none
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \author Fawzi Mohamed
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         SUBROUTINE cp_sll_fm_create(sll, first_el, rest)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            TYPE(cp_sll_fm_type), POINTER :: sll
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            type(cp_fm_type),pointer, OPTIONAL :: first_el
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            TYPE(cp_sll_fm_type), POINTER, OPTIONAL :: rest
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            IF (.NOT. PRESENT(first_el)) THEN
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               NULLIFY (sll)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               IF (PRESENT(rest)) sll => rest
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            ELSE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               ALLOCATE (sll)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               sll%first_el =>first_el
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               NULLIFY (sll%rest)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               IF (PRESENT(rest)) sll%rest => rest
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            END IF
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         END SUBROUTINE cp_sll_fm_create
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \brief deallocates the singly linked list starting at sll.
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      Does not work if loops are present!
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param sll the list to be deallocated
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \par History
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      none
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \author Fawzi Mohamed
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \note
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      does not deallocate the elements that are stored in the list
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      check more?
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         SUBROUTINE cp_sll_fm_dealloc(sll)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            TYPE(cp_sll_fm_type), POINTER :: sll
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            CALL cp_sll_fm_rm_all_el(sll)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         END SUBROUTINE cp_sll_fm_dealloc
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! * low-level *
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \brief deallocates a node of a singly linked list (low level)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param sll the node to be deallocated
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \par History
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      none
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \author Fawzi Mohamed
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         SUBROUTINE cp_sll_fm_dealloc_node(sll)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            TYPE(cp_sll_fm_type), POINTER :: sll
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            DEALLOCATE (sll)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         END SUBROUTINE cp_sll_fm_dealloc_node
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! ============= get/set ============
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \brief returns the first element stored in the list
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param sll the single linked list to get the element from
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \return ...
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \par History
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      none
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \author Fawzi Mohamed
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         FUNCTION cp_sll_fm_get_first_el(sll) RESULT(res)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            TYPE(cp_sll_fm_type), POINTER :: sll
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            type(cp_fm_type),pointer :: res
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            res =>sll%first_el
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         END FUNCTION cp_sll_fm_get_first_el
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \brief returns the rest of the list
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param sll the single linked list to get the rest from
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param iter how many times the call to rest should be iterated,
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>            defaults to 1; -1 means till end of the list.
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \return ...
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \par History
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      none
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \author Fawzi Mohamed
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \note
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      split the case iter=1 to make it more optimized?
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         FUNCTION cp_sll_fm_get_rest(sll, iter) RESULT(res)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            TYPE(cp_sll_fm_type), POINTER :: sll
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            INTEGER, OPTIONAL :: iter
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            TYPE(cp_sll_fm_type), POINTER :: res
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            INTEGER :: i
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            IF (.NOT. ASSOCIATED(sll)) THEN
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               NULLIFY (res)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            ELSE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               IF (PRESENT(iter)) THEN
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                  res => sll
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                  DO i = 1, iter
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                     IF (ASSOCIATED(res%rest)) THEN
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                        res => res%rest
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                     ELSE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                        CPABORT("tried to go past end")
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                     END IF
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                  END DO
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                  IF (iter == -1) THEN
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                     DO
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                        IF (.NOT. ASSOCIATED(res%rest)) EXIT
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                        res => res%rest
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                     END DO
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                  END IF
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               ELSE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                  res => sll%rest ! make the common case fast...
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               END IF
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            END IF
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         END FUNCTION cp_sll_fm_get_rest
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \brief returns the length of the list
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param sll the list you want to know the length of
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \return ...
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \par History
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      none
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \author Fawzi Mohamed
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \note
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      slow (O(n))
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         FUNCTION cp_sll_fm_get_length(sll) RESULT(res)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            TYPE(cp_sll_fm_type), pointer :: sll
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            INTEGER ::res
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            TYPE(cp_sll_fm_type), POINTER :: iterator
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            res = 0
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            iterator => sll
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            DO
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               IF (ASSOCIATED(iterator)) THEN
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                  res = res + 1
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                  iterator => iterator%rest
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               ELSE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                  EXIT
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               END IF
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            END DO
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         END FUNCTION cp_sll_fm_get_length
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \brief returns the element at the given index
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param sll the list you get the element from
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param index the position of the element (stating at 1)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \return ...
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \par History
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      none
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \author Fawzi Mohamed
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \note
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      slow (O(index))
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         FUNCTION cp_sll_fm_get_el_at(sll, index) RESULT(res)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            type(cp_fm_type),pointer :: res
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            TYPE(cp_sll_fm_type), POINTER :: sll
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            INTEGER, INTENT(in) :: index
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            TYPE(cp_sll_fm_type), POINTER :: pos
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            IF (index == -1) THEN
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               pos => cp_sll_fm_get_rest(sll, iter=-1)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            ELSE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               pos => cp_sll_fm_get_rest(sll, iter=index - 1)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            END IF
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            CPASSERT(ASSOCIATED(pos))
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            res =>pos%first_el
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         END FUNCTION cp_sll_fm_get_el_at
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \brief sets the element at the given index
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param sll the list you get the element from
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param index the position of the element (stating at 1)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>             -1 means at the end
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param value the new element
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \par History
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      none
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \author Fawzi Mohamed
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \note
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      slow (O(index))
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         SUBROUTINE cp_sll_fm_set_el_at(sll, index, value)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            type(cp_fm_type),pointer :: value
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            TYPE(cp_sll_fm_type), POINTER :: sll
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            INTEGER, INTENT(in) :: index
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            TYPE(cp_sll_fm_type), POINTER :: pos
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            IF (index == -1) THEN
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               pos => cp_sll_fm_get_rest(sll, iter=-1)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            ELSE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               pos => cp_sll_fm_get_rest(sll, iter=index - 1)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            END IF
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            CPASSERT(ASSOCIATED(pos))
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            pos%first_el =>value
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         END SUBROUTINE cp_sll_fm_set_el_at
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! * iteration *
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \brief returns true if the actual element is valid (i.e. iterator ont at end)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      moves the iterator to the next element
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param iterator iterator that moves along the list
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param el_att the actual element (valid only if the function returns true)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \return ...
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \par History
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      none
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \author Fawzi Mohamed
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         FUNCTION cp_sll_fm_next(iterator, el_att) RESULT(res)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            TYPE(cp_sll_fm_type), POINTER :: iterator
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            type(cp_fm_type),pointer, OPTIONAL :: el_att
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            LOGICAL :: res
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            IF (ASSOCIATED(iterator)) THEN
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               res = .TRUE.
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               if (present(el_att)) el_att =>iterator%first_el
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               iterator => iterator%rest
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            ELSE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               res = .FALSE.
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            END IF
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         END FUNCTION cp_sll_fm_next
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! ============ structure modifications ============
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \brief insert an element at the beginning of the list
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param sll the single linked list point at the beginning of which
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>           you want to add the element
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param el the element to add
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \par History
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      none
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \author Fawzi Mohamed
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \note
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      fast (O(1))
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         SUBROUTINE cp_sll_fm_insert_el(sll, el)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            TYPE(cp_sll_fm_type), POINTER :: sll
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            type(cp_fm_type),pointer:: el
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            TYPE(cp_sll_fm_type), POINTER :: newSlot
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            NULLIFY (newSlot)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            CALL cp_sll_fm_create(newSlot, first_el=el, &
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                                                rest=sll)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            sll => newSlot
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         END SUBROUTINE cp_sll_fm_insert_el
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \brief remove the first element of the linked list
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param sll the list whose first element has to be removed
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \par History
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      none
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \author Fawzi Mohamed
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \note
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      fast (O(1))
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         SUBROUTINE cp_sll_fm_rm_first_el(sll)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            TYPE(cp_sll_fm_type), POINTER :: sll
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            TYPE(cp_sll_fm_type), POINTER :: node_to_rm
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            node_to_rm => sll
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            IF (ASSOCIATED(sll)) THEN
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               sll => sll%rest
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               CALL cp_sll_fm_dealloc_node(node_to_rm)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            ELSE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               CPABORT("tried to remove first el of an empty list")
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            END IF
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         END SUBROUTINE cp_sll_fm_rm_first_el
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \brief inserts the element at the given index
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param sll the list you get the element from
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param el the new element
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param index the position of the element (stating at 1).
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>             If it is -1, it means at end
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \par History
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      none
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \author Fawzi Mohamed
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \note
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      slow (O(index))
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         SUBROUTINE cp_sll_fm_insert_el_at(sll, el, index)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            type(cp_fm_type),pointer :: el
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            INTEGER, INTENT(in) :: index
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            TYPE(cp_sll_fm_type), POINTER :: sll
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            TYPE(cp_sll_fm_type), POINTER :: pos
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            IF (index == 1) THEN
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               CALL cp_sll_fm_insert_el(sll, el)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            ELSE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               IF (index == -1) THEN
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                  pos => cp_sll_fm_get_rest(sll, iter=-1)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               ELSE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                  pos => cp_sll_fm_get_rest(sll, iter=index - 2)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               END IF
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               CPASSERT(ASSOCIATED(pos))
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               CALL cp_sll_fm_insert_el(pos%rest, el)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            END IF
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         END SUBROUTINE cp_sll_fm_insert_el_at
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \brief removes the element at the given index
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param sll the list you get the element from
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param index the position of the element (stating at 1)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \par History
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      none
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \author Fawzi Mohamed
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \note
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      slow (O(index))
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         SUBROUTINE cp_sll_fm_rm_el_at(sll, index)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            TYPE(cp_sll_fm_type), POINTER :: sll
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            INTEGER, INTENT(in)::index
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            TYPE(cp_sll_fm_type), POINTER :: pos
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            IF (index == 1) THEN
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               CALL cp_sll_fm_rm_first_el(sll)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            ELSE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               IF (index == -1) THEN
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                  pos => cp_sll_fm_get_rest(sll, iter=-1)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               ELSE
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
                  pos => cp_sll_fm_get_rest(sll, iter=index - 2)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               END IF
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               CPASSERT(ASSOCIATED(pos))
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               CALL cp_sll_fm_rm_first_el(pos%rest)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            END IF
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         END SUBROUTINE cp_sll_fm_rm_el_at
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \brief removes all the elements from the list
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param sll the list that should be removed
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \par History
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      none
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \author Fawzi Mohamed
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \note
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      check more?
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         SUBROUTINE cp_sll_fm_rm_all_el(sll)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            TYPE(cp_sll_fm_type), POINTER :: sll
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            TYPE(cp_sll_fm_type), POINTER :: next_node, actual_node
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            actual_node => sll
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            DO
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               IF (.NOT. ASSOCIATED(actual_node)) EXIT
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               next_node => actual_node%rest
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               CALL cp_sll_fm_dealloc_node(actual_node)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               actual_node => next_node
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            END DO
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            NULLIFY (sll)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         END SUBROUTINE cp_sll_fm_rm_all_el
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \brief returns a newly allocated array with the same contents as
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      the linked list
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \param sll the list to transform in array
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \return ...
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \par History
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!>      07.2002 created [fawzi]
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
!> \author Fawzi Mohamed
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
! **************************************************************************************************
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         FUNCTION cp_sll_fm_to_array(sll) RESULT(res)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            TYPE(cp_sll_fm_type), POINTER :: sll
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            type(cp_fm_p_type), DIMENSION(:), POINTER :: res
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            INTEGER :: len, i
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            LOGICAL :: ok
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            TYPE(cp_sll_fm_type), POINTER :: iter
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"

# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            len = cp_sll_fm_get_length(sll)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            ALLOCATE (res(len))
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            iter => sll
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            DO i = 1, len
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               res(i) %matrix=>iter%first_el
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               ok = cp_sll_fm_next(iter)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
               CPASSERT(ok .OR. i == len)
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
            END DO
# 27 "/builddir/build/BUILD/cp2k-9.1/src/fm/cp_linked_list_fm.F"
         END FUNCTION cp_sll_fm_to_array
END MODULE
